@@ -14,6 +14,27 @@ from locale import LC_NUMERIC, atof, setlocale
 
 from openpyxl import load_workbook
 
+SUPPORTED_OUTPUT_FORMATS = None
+
+
+def define_supported_output_formats():
+    global SUPPORTED_OUTPUT_FORMATS
+    SUPPORTED_OUTPUT_FORMATS = [
+        "Integer",
+        "Decimal",
+        "Keep numeric",
+        "Time",
+        "Text",
+    ]
+    delimiters = ["/", "-", ".", ""]
+    date_patterns = ["YYYYMMDD"]
+    for d in delimiters:
+        date_patterns.append("MM{0}DD{0}YYYY".format(d))
+        date_patterns.append("DD{0}MM{0}YYYY".format(d))
+    for p1 in date_patterns:
+        for p2 in date_patterns:
+            SUPPORTED_OUTPUT_FORMATS.append("Date ({0} to {1})".format(p2, p1))
+
 
 def write_output_file(output_content, output_file):
     with open(output_file, "w") as ofile:
@@ -21,7 +42,7 @@ def write_output_file(output_content, output_file):
 
 
 def pad_output_value(val, output_format, length):
-    if output_format in ("Integer", "Decimal"):
+    if output_format in ("Integer", "Decimal", "Keep numeric"):
         # Numbers get padded with 0's added in front (to the left)
         val = str(val).zfill(length)
     else:
@@ -31,26 +52,45 @@ def pad_output_value(val, output_format, length):
     return val
 
 
-def convert_date(value, output_format, idx_col, idx_row):
-    converted_value = ""
-    delim = output_format[8]
-    if delim not in ("/", "-", "."):
-        delim = ""
-    if delim == "" and len(value) != 8:
-        logging.critical(
-            "Invalid date value '%s' for format '%s' in field %d on row %d "
-            "(ignoring the header), day and month must contain leading 0's. Exiting..."
-            % (value, output_format, idx_col, idx_row)
-        )
-        sys.exit(24)
+def determine_date_delimiters(output_format):
+    # Identify the delimiter in the input format
+    supported_delimiters = ("/", "-", ".")
+    in_delim = output_format[8]
+    if in_delim not in supported_delimiters:
+        in_delim = ""
+
+    # Identify the delimiter in the output format
+    out_delim = output_format[-4]
+    if out_delim == "Y":
+        out_delim = output_format[-6]
+    if out_delim not in supported_delimiters:
+        out_delim = ""
+    return (in_delim, out_delim)
+
+
+def parse_input_date(value, output_format, in_delim):
+    m = None
     if output_format.startswith("Date (DD"):
         m = re.match(
-            r"([0123]?\d){delim}([01]?\d){delim}(\d{{4}})".format(delim=delim), value
+            r"([0123]?\d){in_delim}([01]?\d){in_delim}(\d{{4}})".format(
+                in_delim=in_delim
+            ),
+            value,
         )
     elif output_format.startswith("Date (MM"):
         m = re.match(
-            r"([01]?\d){delim}([0123]?\d){delim}(\d{{4}})".format(delim=delim), value
+            r"([01]?\d){in_delim}([0123]?\d){in_delim}(\d{{4}})".format(
+                in_delim=in_delim
+            ),
+            value,
         )
+    elif output_format.startswith("Date (YYYY"):
+        m = re.match(
+            r"(\d{{4}}){in_delim}([0123]?\d)([01]?\d)".format(in_delim=in_delim), value
+        )
+    year = None
+    month = None
+    day = None
     if m:
         if output_format.startswith("Date (DD"):
             year = m.group(3)
@@ -60,12 +100,52 @@ def convert_date(value, output_format, idx_col, idx_row):
             year = m.group(3)
             month = m.group(1).zfill(2)
             day = m.group(2).zfill(2)
-        converted_value = "%s%s%s" % (year, month, day)
-        # Is it a valid date?
-        try:
-            datetime.datetime.strptime(converted_value, "%Y%m%d")
-        except ValueError:
-            converted_value = ""
+        elif output_format.startswith("Date (YYYY"):
+            year = m.group(1)
+            month = m.group(2).zfill(2)
+            day = m.group(3).zfill(2)
+    return (year, month, day)
+
+
+def generate_output_date(year, month, day, output_format, out_delim):
+    converted_value = ""
+    if output_format.endswith(
+        " to YYYY{out_delim}MM{out_delim}DD)".format(out_delim=out_delim)
+    ):
+        converted_value = "%s%s%s%s%s" % (year, out_delim, month, out_delim, day)
+    if output_format.endswith(
+        " to DD{out_delim}MM{out_delim}YYYY)".format(out_delim=out_delim)
+    ):
+        converted_value = "%s%s%s%s%s" % (day, out_delim, month, out_delim, year)
+    if output_format.endswith(
+        " to MM{out_delim}DD{out_delim}YYYY)".format(out_delim=out_delim)
+    ):
+        converted_value = "%s%s%s%s%s" % (month, out_delim, day, out_delim, year)
+    return converted_value
+
+
+def convert_date(value, output_format, idx_col, idx_row):
+    converted_value = ""
+    (in_delim, out_delim) = determine_date_delimiters(output_format)
+    if in_delim == "" and len(value) != 8:
+        logging.critical(
+            "Invalid date value '%s' for format '%s' in field %d on row %d "
+            "(ignoring the header), day and month must contain leading 0's. Exiting..."
+            % (value, output_format, idx_col, idx_row)
+        )
+        sys.exit(24)
+
+    (year, month, day) = parse_input_date(value, output_format, in_delim)
+
+    # Is it a valid date?
+    try:
+        datetime.datetime.strptime("%s%s%s" % (year, month, day), "%Y%m%d")
+    except ValueError:
+        pass
+    else:
+        converted_value = generate_output_date(
+            year, month, day, output_format, out_delim
+        )
     if not converted_value:
         logging.critical(
             "Invalid date value '%s' for format '%s' in field %d on row %d "
@@ -77,8 +157,24 @@ def convert_date(value, output_format, idx_col, idx_row):
 
 
 def convert_cell(value, output_format, idx_col, idx_row):
+    if output_format not in SUPPORTED_OUTPUT_FORMATS:
+        logging.critical(
+            "Invalid output format '%s', must be one of '%s'. "
+            "Exiting..."
+            % (
+                output_format,
+                "', '".join(SUPPORTED_OUTPUT_FORMATS),
+            )
+        )
+        sys.exit(27)
     converted_value = ""
-    if output_format in ("Integer", "Decimal") and str(value).strip() == "":
+    if "Keep numeric" == output_format:
+        # Strip all non-numeric characters
+        value = re.sub(r"\D", "", value)
+    if (
+        output_format in ("Integer", "Decimal", "Keep numeric")
+        and str(value).strip() == ""
+    ):
         value = "0"
     if "Time" == output_format:
         m = re.match(r"(\d{2})(:)?(\d{2})", value)
@@ -118,8 +214,11 @@ def convert_cell(value, output_format, idx_col, idx_row):
     return converted_value
 
 
-def convert_content(input_content, config, date_field_to_report_on=None, truncate=None):
+def convert_content(
+    input_content, config, date_field_to_report_on=None, truncate=None, divert=None
+):
     output_content = []
+    diverted_output_content = []
     if date_field_to_report_on:
         # Argument is 1-based
         date_field_to_report_on -= 1
@@ -127,6 +226,7 @@ def convert_content(input_content, config, date_field_to_report_on=None, truncat
     most_recent_date = "00000000"
     for idx_row, row in enumerate(input_content):
         converted_row_content = []
+        divert_row = False
         # Confirm that the input_content doesn't have more fields than are
         # defined in the configuration file
         if len(row) > len(config):
@@ -141,6 +241,11 @@ def convert_content(input_content, config, date_field_to_report_on=None, truncat
             output_format = config[idx_col]["output_format"]
             length = config[idx_col]["length"]
 
+            if divert and idx_col + 1 in divert and str(cell) in divert[idx_col + 1]:
+                # This field contains a value marked for content diversion
+                # The content for the entire row will be diverted to a separate file
+                divert_row = True
+
             if config[idx_col]["skip_field"]:
                 cell = ""
             else:
@@ -190,10 +295,13 @@ def convert_content(input_content, config, date_field_to_report_on=None, truncat
             length = config[idx_col]["length"]
             padded_output_value = pad_output_value("", output_format, length)
             converted_row_content.append(padded_output_value)
-        output_content.append("".join(converted_row_content))
+        if divert_row:
+            diverted_output_content.append("".join(converted_row_content))
+        else:
+            output_content.append("".join(converted_row_content))
 
     logging.debug("The output content:\n%s" % "\n".join(output_content))
-    return (output_content, oldest_date, most_recent_date)
+    return (output_content, diverted_output_content, oldest_date, most_recent_date)
 
 
 def read_input_file(input_file, delimiter, quotechar, skip_header, skip_footer):
@@ -218,20 +326,6 @@ def read_input_file(input_file, delimiter, quotechar, skip_header, skip_footer):
 
 def load_config(config_file):
     config = []
-    supported_output_formats = (
-        "Integer",
-        "Decimal",
-        "Time",
-        "Text",
-        "Date (DD/MM/YYYY to YYYYMMDD)",
-        "Date (DD-MM-YYYY to YYYYMMDD)",
-        "Date (DD.MM.YYYY to YYYYMMDD)",
-        "Date (DDMMYYYY to YYYYMMDD)",
-        "Date (MM/DD/YYYY to YYYYMMDD)",
-        "Date (MM-DD-YYYY to YYYYMMDD)",
-        "Date (MM.DD.YYYY to YYYYMMDD)",
-        "Date (MMDDYYYY to YYYYMMDD)",
-    )
     supported_skip_field = ("True", "False", "", None)
     logging.debug("Loading configuration %s" % config_file)
 
@@ -276,7 +370,7 @@ def load_config(config_file):
                     )
                     sys.exit(14)
             if idx_col == output_format_col:
-                if cell.value in supported_output_formats:
+                if cell.value in SUPPORTED_OUTPUT_FORMATS:
                     config[idx_row]["output_format"] = cell.value
                 else:
                     logging.critical(
@@ -285,7 +379,7 @@ def load_config(config_file):
                         % (
                             cell.value,
                             idx_row + 2,
-                            "', '".join(supported_output_formats),
+                            "', '".join(SUPPORTED_OUTPUT_FORMATS),
                         )
                     )
                     sys.exit(15)
@@ -314,6 +408,32 @@ def get_version(rel_path):
             raise RuntimeError("Unable to find version string.")
 
 
+def validate_divert(divert):
+    divert_values = {}
+    for d in divert:
+        v = d.split(",", 1)
+        try:
+            v[0] = int(v[0])
+        except ValueError:
+            logging.critical(
+                "<field number> must be a number, as passed to the `--divert` "
+                'argument in the format "<field number>,<value to divert on>" '
+                "(without quotes). Exiting..."
+            )
+            sys.exit(28)
+        if len(v) == 2:
+            if v[0] not in divert_values:
+                divert_values[v[0]] = []
+            divert_values[v[0]].append(v[1])
+        else:
+            logging.critical(
+                'The `--divert` argument must be formatted as "<field number>,'
+                '<value to divert on>" (without quotes). Exiting...'
+            )
+            sys.exit(29)
+    return divert_values
+
+
 def validate_shared_args(args):
     if not os.path.isfile(args.input):
         logging.critical("The specified input file does not exist. Exiting...")
@@ -345,6 +465,8 @@ def validate_shared_args(args):
                 )
                 sys.exit(25)
         args.truncate = truncate
+    if args.divert:
+        args.divert = validate_divert(args.divert)
 
 
 def add_shared_args(parser):
@@ -409,6 +531,19 @@ def add_shared_args(parser):
         required=False,
         default=[],
     )
+    parser.add_argument(
+        "-dv",
+        "--divert",
+        help="Diverts to a separate file the content from rows containing a specific "
+        'value at a specific place. The format of this parameter is "<field number>'
+        ',<value to divert on>" (without quotes). This parameter can be repeated '
+        "several times to support different values or different fields. The diverted "
+        "content will be saved to a file whose name will be the output filename with "
+        '"_diverted" added before the file extension.',
+        action="append",
+        required=False,
+        default=[],
+    )
 
 
 def parse_args(arguments):
@@ -482,11 +617,14 @@ def process(
     date_field_to_report_on=None,
     locale="",
     truncate=None,
+    divert=None,
 ):
     # By default, set to the user's default locale, used to appropriately handle
     # Decimal separators
     setlocale(LC_NUMERIC, locale)
 
+    define_supported_output_formats()
+
     config = load_config(config)
 
     if truncate:
@@ -498,16 +636,35 @@ def process(
                     "Exiting..." % (t, len(config))
                 )
                 sys.exit(26)
+    if divert:
+        for d in divert.keys():
+            if d > len(config):
+                logging.critical(
+                    "The value %d passed as field ID in the `--divert` argument is "
+                    "invalid, it is higher than the %d fields defined in the "
+                    "configuration file. Exiting..." % (d, len(config))
+                )
+                sys.exit(30)
 
     input_content = read_input_file(
         input, delimiter, quotechar, skip_header, skip_footer
     )
 
-    (output_content, oldest_date, most_recent_date) = convert_content(
-        input_content, config, date_field_to_report_on, truncate
+    (
+        output_content,
+        diverted_output_content,
+        oldest_date,
+        most_recent_date,
+    ) = convert_content(
+        input_content, config, date_field_to_report_on, truncate, divert
     )
 
     write_output_file(output_content, output)
+    if diverted_output_content:
+        # Save the diverted content to its separate file with "_diverted" added before
+        # the extension
+        diverted_output = "%s_diverted%s" % (os.path.splitext(output))
+        write_output_file(diverted_output_content, diverted_output)
 
     return (len(input_content), oldest_date, most_recent_date)
 
@@ -528,6 +685,7 @@ def init():
             None,
             args.locale,
             args.truncate,
+            args.divert,
         )
 
 
