@@ -9,19 +9,115 @@
 #   --title="Code test coverage for delimited2fixedwidth"
 
 import contextlib
+import datetime
 import io
 import logging
 import os
+import re
 import sys
 import tempfile
 import unittest
+from locale import LC_NUMERIC
+from locale import Error as localeError
+from locale import getlocale, setlocale
 
-CURRENT_VERSION = "1.0.4"
+CURRENT_VERSION = "1.0.7"
 
 sys.path.append(".")
 target = __import__("delimited2fixedwidth")
 
 
+def get_expected_supported_output_formats():
+    return [
+        "Integer",
+        "Decimal",
+        "Keep numeric",
+        "Time",
+        "Text",
+        "Date (YYYYMMDD to YYYYMMDD)",
+        "Date (MM/DD/YYYY to YYYYMMDD)",
+        "Date (DD/MM/YYYY to YYYYMMDD)",
+        "Date (MM-DD-YYYY to YYYYMMDD)",
+        "Date (DD-MM-YYYY to YYYYMMDD)",
+        "Date (MM.DD.YYYY to YYYYMMDD)",
+        "Date (DD.MM.YYYY to YYYYMMDD)",
+        "Date (MMDDYYYY to YYYYMMDD)",
+        "Date (DDMMYYYY to YYYYMMDD)",
+        "Date (YYYYMMDD to MM/DD/YYYY)",
+        "Date (MM/DD/YYYY to MM/DD/YYYY)",
+        "Date (DD/MM/YYYY to MM/DD/YYYY)",
+        "Date (MM-DD-YYYY to MM/DD/YYYY)",
+        "Date (DD-MM-YYYY to MM/DD/YYYY)",
+        "Date (MM.DD.YYYY to MM/DD/YYYY)",
+        "Date (DD.MM.YYYY to MM/DD/YYYY)",
+        "Date (MMDDYYYY to MM/DD/YYYY)",
+        "Date (DDMMYYYY to MM/DD/YYYY)",
+        "Date (YYYYMMDD to DD/MM/YYYY)",
+        "Date (MM/DD/YYYY to DD/MM/YYYY)",
+        "Date (DD/MM/YYYY to DD/MM/YYYY)",
+        "Date (MM-DD-YYYY to DD/MM/YYYY)",
+        "Date (DD-MM-YYYY to DD/MM/YYYY)",
+        "Date (MM.DD.YYYY to DD/MM/YYYY)",
+        "Date (DD.MM.YYYY to DD/MM/YYYY)",
+        "Date (MMDDYYYY to DD/MM/YYYY)",
+        "Date (DDMMYYYY to DD/MM/YYYY)",
+        "Date (YYYYMMDD to MM-DD-YYYY)",
+        "Date (MM/DD/YYYY to MM-DD-YYYY)",
+        "Date (DD/MM/YYYY to MM-DD-YYYY)",
+        "Date (MM-DD-YYYY to MM-DD-YYYY)",
+        "Date (DD-MM-YYYY to MM-DD-YYYY)",
+        "Date (MM.DD.YYYY to MM-DD-YYYY)",
+        "Date (DD.MM.YYYY to MM-DD-YYYY)",
+        "Date (MMDDYYYY to MM-DD-YYYY)",
+        "Date (DDMMYYYY to MM-DD-YYYY)",
+        "Date (YYYYMMDD to DD-MM-YYYY)",
+        "Date (MM/DD/YYYY to DD-MM-YYYY)",
+        "Date (DD/MM/YYYY to DD-MM-YYYY)",
+        "Date (MM-DD-YYYY to DD-MM-YYYY)",
+        "Date (DD-MM-YYYY to DD-MM-YYYY)",
+        "Date (MM.DD.YYYY to DD-MM-YYYY)",
+        "Date (DD.MM.YYYY to DD-MM-YYYY)",
+        "Date (MMDDYYYY to DD-MM-YYYY)",
+        "Date (DDMMYYYY to DD-MM-YYYY)",
+        "Date (YYYYMMDD to MM.DD.YYYY)",
+        "Date (MM/DD/YYYY to MM.DD.YYYY)",
+        "Date (DD/MM/YYYY to MM.DD.YYYY)",
+        "Date (MM-DD-YYYY to MM.DD.YYYY)",
+        "Date (DD-MM-YYYY to MM.DD.YYYY)",
+        "Date (MM.DD.YYYY to MM.DD.YYYY)",
+        "Date (DD.MM.YYYY to MM.DD.YYYY)",
+        "Date (MMDDYYYY to MM.DD.YYYY)",
+        "Date (DDMMYYYY to MM.DD.YYYY)",
+        "Date (YYYYMMDD to DD.MM.YYYY)",
+        "Date (MM/DD/YYYY to DD.MM.YYYY)",
+        "Date (DD/MM/YYYY to DD.MM.YYYY)",
+        "Date (MM-DD-YYYY to DD.MM.YYYY)",
+        "Date (DD-MM-YYYY to DD.MM.YYYY)",
+        "Date (MM.DD.YYYY to DD.MM.YYYY)",
+        "Date (DD.MM.YYYY to DD.MM.YYYY)",
+        "Date (MMDDYYYY to DD.MM.YYYY)",
+        "Date (DDMMYYYY to DD.MM.YYYY)",
+        "Date (YYYYMMDD to MMDDYYYY)",
+        "Date (MM/DD/YYYY to MMDDYYYY)",
+        "Date (DD/MM/YYYY to MMDDYYYY)",
+        "Date (MM-DD-YYYY to MMDDYYYY)",
+        "Date (DD-MM-YYYY to MMDDYYYY)",
+        "Date (MM.DD.YYYY to MMDDYYYY)",
+        "Date (DD.MM.YYYY to MMDDYYYY)",
+        "Date (MMDDYYYY to MMDDYYYY)",
+        "Date (DDMMYYYY to MMDDYYYY)",
+        "Date (YYYYMMDD to DDMMYYYY)",
+        "Date (MM/DD/YYYY to DDMMYYYY)",
+        "Date (DD/MM/YYYY to DDMMYYYY)",
+        "Date (MM-DD-YYYY to DDMMYYYY)",
+        "Date (DD-MM-YYYY to DDMMYYYY)",
+        "Date (MM.DD.YYYY to DDMMYYYY)",
+        "Date (DD.MM.YYYY to DDMMYYYY)",
+        "Date (MMDDYYYY to DDMMYYYY)",
+        "Date (DDMMYYYY to DDMMYYYY)",
+    ]
+
+
 class TestWriteOutputFile(unittest.TestCase):
     def test_write_output_file(self):
         """
@@ -40,6 +136,10 @@ class TestWriteOutputFile(unittest.TestCase):
 
 
 class TestLoadConfig(unittest.TestCase):
+    @classmethod
+    def setUpClass(cls):
+        target.define_supported_output_formats()
+
     def test_load_config_valid(self):
         """
         Test loading a valid configuration file
@@ -112,10 +212,9 @@ class TestLoadConfig(unittest.TestCase):
         self.assertEqual(
             cm2.output,
             [
-                "CRITICAL:root:Invalid output format 'INVALID OUTPUT FORMAT' on "
-                "row 9, must be one of 'Integer', 'Decimal', 'Time', 'Text', "
-                "'Date (DD/MM/YYYY to YYYYMMDD)', 'Date (MM/DD/YYYY to YYYYMMDD)'. "
-                "Exiting..."
+                "CRITICAL:root:Invalid output format 'INVALID OUTPUT FORMAT' on row 9, "
+                "must be one of '%s'. Exiting..."
+                % "', '".join(get_expected_supported_output_formats())
             ],
         )
 
@@ -202,12 +301,16 @@ class TestReadInputFile(unittest.TestCase):
 
 
 class TestConvertContent(unittest.TestCase):
+    @classmethod
+    def setUpClass(cls):
+        target.define_supported_output_formats()
+
     def test_convert_content_empty(self):
         """
         Test converting empty full content
         """
         output_content = target.convert_content([], None)
-        self.assertEqual(output_content, ([], "99999999", "00000000"))
+        self.assertEqual(output_content, ([], [], "99999999", "00000000"))
 
     def test_convert_content_valid(self):
         """
@@ -227,9 +330,7 @@ class TestConvertContent(unittest.TestCase):
                 "skip_field": False,
             },
         ]
-        (output_content, ignore1, ignore2) = target.convert_content(
-            input_content, config
-        )
+        (output_content, _, _, _) = target.convert_content(input_content, config)
         expected_output = [
             "0142This is just text   20200620",
             "2247Short text          20201129",
@@ -267,6 +368,33 @@ class TestConvertContent(unittest.TestCase):
             ],
         )
 
+    def test_convert_content_field_too_long_truncate(self):
+        """
+        Test converting full content with one field that's too long
+        """
+        input_content = [
+            ["2247", "Short text", "not important", "29/11/2020"],
+            ["01:42", "This text is too long", "blabla", "20/6/2020"],
+        ]
+        config = [
+            {"length": 4, "output_format": "Time", "skip_field": False},
+            {"length": 20, "output_format": "Text", "skip_field": False},
+            {"length": 0, "output_format": "Text", "skip_field": True},
+            {
+                "length": 8,
+                "output_format": "Date (DD/MM/YYYY to YYYYMMDD)",
+                "skip_field": False,
+            },
+        ]
+        (output_content, _, _, _) = target.convert_content(
+            input_content, config, None, [2, 3]
+        )
+        expected_output = [
+            "2247Short text          20201129",
+            "0142This text is too lon20200620",
+        ]
+        self.assertEqual(output_content, expected_output)
+
     def test_convert_content_too_many_input_fields(self):
         """
         Test converting full content where the input data has more fields than
@@ -321,9 +449,7 @@ class TestConvertContent(unittest.TestCase):
             },
             {"length": 10, "output_format": "Integer", "skip_field": False},
         ]
-        (output_content, ignore1, ignore2) = target.convert_content(
-            input_content, config
-        )
+        (output_content, _, _, _) = target.convert_content(input_content, config)
         expected_output = [
             "0142This is just text   202006200000000000",
             "2247Short text                  0000000000",
@@ -352,7 +478,7 @@ class TestConvertContent(unittest.TestCase):
             },
         ]
         date_field_to_report_on = 4
-        (output_content, oldest_date, most_recent_date) = target.convert_content(
+        (output_content, _, oldest_date, most_recent_date) = target.convert_content(
             input_content, config, date_field_to_report_on
         )
         expected_output = [
@@ -366,8 +492,64 @@ class TestConvertContent(unittest.TestCase):
         self.assertEqual(oldest_date, "20200620")
         self.assertEqual(most_recent_date, "20201129")
 
+    def test_convert_content_divert(self):
+        """
+        Test converting full content where the input data has less fields than
+        are defined in the configuration
+        """
+        input_content = [
+            ["01:42", "This is just text", "blabla", 1, "20/6/2020"],
+            ["1357", "Hello", "ignore", 2],
+            ["2247", "Short text", "not important", 1],
+            ["0934", "Hello again", "nope", 1, "29/11/2020", "divert"],
+        ]
+        config = [
+            {"length": 4, "output_format": "Time", "skip_field": False},
+            {"length": 20, "output_format": "Text", "skip_field": False},
+            {"length": 0, "output_format": "Text", "skip_field": True},
+            {"length": 0, "output_format": "Text", "skip_field": True},
+            {
+                "length": 8,
+                "output_format": "Date (DD/MM/YYYY to YYYYMMDD)",
+                "skip_field": False,
+            },
+            {"length": 0, "output_format": "Text", "skip_field": True},
+            {"length": 10, "output_format": "Integer", "skip_field": False},
+        ]
+        divert_arg = {4: ["2", "3", "4"], 6: ["divert"]}
+        (output_content, diverted_output_content, _, _) = target.convert_content(
+            input_content, config, None, None, divert_arg
+        )
+        expected_output = [
+            "0142This is just text   202006200000000000",
+            "2247Short text                  0000000000",
+        ]
+        self.assertEqual(output_content, expected_output)
+        expected_diverted_output_content = [
+            "1357Hello                       0000000000",
+            "0934Hello again         202011290000000000",
+        ]
+        self.assertEqual(diverted_output_content, expected_diverted_output_content)
+
+
+class TestDefineSupportedOutputFormats(unittest.TestCase):
+    def test_define_supported_output_formats(self):
+        """
+        Test defining the supported output formats
+        """
+
+        target.define_supported_output_formats()
+        self.assertEqual(
+            target.SUPPORTED_OUTPUT_FORMATS,
+            get_expected_supported_output_formats(),
+        )
+
 
 class TestConvertCell(unittest.TestCase):
+    @classmethod
+    def setUpClass(cls):
+        target.define_supported_output_formats()
+
     def test_convert_cell_time_colon(self):
         """
         Test converting a valid time element with a colon separator
@@ -416,39 +598,82 @@ class TestConvertCell(unittest.TestCase):
             ],
         )
 
-    def test_convert_cell_date_ddmmyyyy_slashes(self):
-        """
-        Test converting a valid date value with format DD/MM/YYYY
-        """
-        date = "03/11/1981"
-        output_value = target.convert_cell(date, "Date (DD/MM/YYYY to YYYYMMDD)", 2, 3)
-        self.assertEqual(output_value, "19811103")
+    def test_convert_cell_date_global(self):
+        year = "1981"
+        month = "11"
+        day = "03"
+        for output_format in target.SUPPORTED_OUTPUT_FORMATS:
+            # print(year, month, day, output_format)
+            m = re.match(r"Date \((.*) to (.*)\)", output_format)
+            if m:
+                in_format = m.group(1)
+                out_format = m.group(2)
+                # print(in_format, out_format)
+                in_format = (
+                    in_format.replace("YYYY", "%Y")
+                    .replace("MM", "%m")
+                    .replace("DD", "%d")
+                )
+                out_format = (
+                    out_format.replace("YYYY", "%Y")
+                    .replace("MM", "%m")
+                    .replace("DD", "%d")
+                )
+                # print(in_format, out_format)
+                year = int(year)
+                month = int(month)
+                day = int(day)
+                in_value = datetime.date(year, month, day).strftime(in_format)
+                expected_out_value = datetime.date(year, month, day).strftime(
+                    out_format
+                )
+                # print(in_value, expected_out_value)
+                output_value = target.convert_cell(in_value, output_format, 2, 3)
+                self.assertEqual(output_value, expected_out_value)
+            else:
+                # Not a date format, pass over it
+                pass
+            # print()
 
-    def test_convert_cell_date_mmddyyyy_slashes(self):
-        """
-        Test converting a valid date value with format MM/DD/YYYY
-        """
-        date = "11/03/1981"
-        output_value = target.convert_cell(date, "Date (MM/DD/YYYY to YYYYMMDD)", 2, 3)
-        self.assertEqual(output_value, "19811103")
-
-    def test_convert_cell_date_ddmmyyyy_slashes_short(self):
+    def test_convert_cell_date_ddmmyyyy_no_delimiter_short_invalid(self):
         """
         Test converting a valid date value with format DD/MM/YYYY, with
         single-digit day and month
         """
-        date = "3/1/1981"
-        output_value = target.convert_cell(date, "Date (DD/MM/YYYY to YYYYMMDD)", 2, 3)
-        self.assertEqual(output_value, "19810103")
+        date = "311981"
+        with self.assertRaises(SystemExit) as cm1, self.assertLogs(
+            level="CRITICAL"
+        ) as cm2:
+            target.convert_cell(date, "Date (DDMMYYYY to YYYYMMDD)", 2, 3)
+        self.assertEqual(cm1.exception.code, 24)
+        self.assertEqual(
+            cm2.output,
+            [
+                "CRITICAL:root:Invalid date value '311981' for format 'Date "
+                "(DDMMYYYY to YYYYMMDD)' in field 2 on row 3 (ignoring the "
+                "header), day and month must contain leading 0's. Exiting..."
+            ],
+        )
 
-    def test_convert_cell_date_mmddyyyy_slashes_short(self):
+    def test_convert_cell_date_mmddyyyy_no_delimiter_short_invalid(self):
         """
-        Test converting a valid date value with format MM/DD/YYYY, with
+        Test converting a valid date value with format MMDDYYYY, with
         single-digit day and month
         """
-        date = "1/3/1981"
-        output_value = target.convert_cell(date, "Date (MM/DD/YYYY to YYYYMMDD)", 2, 3)
-        self.assertEqual(output_value, "19810103")
+        date = "131981"
+        with self.assertRaises(SystemExit) as cm1, self.assertLogs(
+            level="CRITICAL"
+        ) as cm2:
+            target.convert_cell(date, "Date (MMDDYYYY to YYYYMMDD)", 2, 3)
+        self.assertEqual(cm1.exception.code, 24)
+        self.assertEqual(
+            cm2.output,
+            [
+                "CRITICAL:root:Invalid date value '131981' for format 'Date "
+                "(MMDDYYYY to YYYYMMDD)' in field 2 on row 3 (ignoring the "
+                "header), day and month must contain leading 0's. Exiting..."
+            ],
+        )
 
     def test_convert_cell_date_ddmmyyyy_slashes_invalid_date(self):
         """
@@ -526,6 +751,34 @@ class TestConvertCell(unittest.TestCase):
             ],
         )
 
+    def test_convert_cell_integer(self):
+        """
+        Test converting a valid integer value.
+        """
+        output_value = target.convert_cell(136, "Integer", 2, 3)
+        self.assertEqual(output_value, 136)
+
+    def test_convert_cell_integer_as_string(self):
+        """
+        Test converting a valid integer value, sent as string
+        """
+        output_value = target.convert_cell("136", "Integer", 2, 3)
+        self.assertEqual(output_value, "136")
+
+    def test_convert_cell_integer_empty_string(self):
+        """
+        Test converting a integer value that was an empty string
+        """
+        output_value = target.convert_cell("", "Integer", 2, 5)
+        self.assertEqual(output_value, "0")
+
+    def test_convert_cell_integer_spaces(self):
+        """
+        Test converting a integer value that are just spaces
+        """
+        output_value = target.convert_cell("   ", "Integer", 2, 5)
+        self.assertEqual(output_value, "0")
+
     def test_convert_cell_decimal(self):
         """
         Test converting a valid decimal value.
@@ -542,6 +795,46 @@ class TestConvertCell(unittest.TestCase):
         output_value = target.convert_cell(2, "Decimal", 2, 3)
         self.assertEqual(output_value, "200")
 
+    def test_convert_cell_decimal_as_string(self):
+        """
+        Test converting a valid decimal value, sent as string
+        Returns "cents" instead of "dollars"
+        """
+        output_value = target.convert_cell("1.36", "Decimal", 2, 3)
+        self.assertEqual(output_value, "136")
+
+    def test_convert_cell_decimal_with_commas(self):
+        """
+        Test converting a valid decimal value, sent with a comma as decimal separator.
+        Let's test with the French locale.
+        Returns "cents" instead of "dollars"
+        """
+        # Save default locale
+        loc = getlocale(LC_NUMERIC)
+        # Set to French locale
+        try:
+            setlocale(LC_NUMERIC, "fr_FR.utf8")
+        except localeError:
+            setlocale(LC_NUMERIC, "fr")
+        output_value = target.convert_cell("1,36", "Decimal", 2, 3)
+        self.assertEqual(output_value, "136")
+        # Revert back to default locale
+        setlocale(LC_NUMERIC, loc)
+
+    def test_convert_cell_decimal_empty_string(self):
+        """
+        Test converting a decimal value that was an empty string
+        """
+        output_value = target.convert_cell("", "Decimal", 2, 5)
+        self.assertEqual(output_value, "0")
+
+    def test_convert_cell_decimal_spaces(self):
+        """
+        Test converting a decimal value that are just spaces
+        """
+        output_value = target.convert_cell("   ", "Decimal", 2, 5)
+        self.assertEqual(output_value, "0")
+
     def test_convert_cell_decimal_rounding(self):
         """
         Test converting a valid decimal value, rounding the value
@@ -567,6 +860,20 @@ class TestConvertCell(unittest.TestCase):
             ],
         )
 
+    def test_convert_cell_keep_numeric(self):
+        """
+        Test converting a value to keep only the numeric elements
+        """
+        output_value = target.convert_cell("00.00.00-000.00a", "Keep numeric", 2, 3)
+        self.assertEqual(output_value, "00000000000")
+
+    def test_convert_cell_keep_numeric_empty_string(self):
+        """
+        Test converting a keep_numeric value that was an empty string
+        """
+        output_value = target.convert_cell("", "Keep numeric", 2, 5)
+        self.assertEqual(output_value, "0")
+
     def test_convert_cell_text(self):
         """
         Test converting a valid text value, returns the same value
@@ -576,11 +883,20 @@ class TestConvertCell(unittest.TestCase):
 
     def test_convert_cell_text_nonsense_output_format(self):
         """
-        Test converting a valid text value, passing a nonsense output_format.
-        Returns the same value
+        Test converting a valid text value, passing a nonsense output_format
         """
-        output_value = target.convert_cell("This is the value", "blabla", 2, 3)
-        self.assertEqual(output_value, "This is the value")
+        with self.assertRaises(SystemExit) as cm1, self.assertLogs(
+            level="CRITICAL"
+        ) as cm2:
+            target.convert_cell("This is the value", "blabla", 2, 3)
+        self.assertEqual(cm1.exception.code, 27)
+        self.assertEqual(
+            cm2.output,
+            [
+                "CRITICAL:root:Invalid output format 'blabla', must be one of '%s'. "
+                "Exiting..." % "', '".join(get_expected_supported_output_formats())
+            ],
+        )
 
 
 class TestPadOutputValue(unittest.TestCase):
@@ -718,19 +1034,42 @@ class TestParseArgs(unittest.TestCase):
             self.assertFalse(os.path.isfile(output_file))
         with self.assertLogs(level="DEBUG") as cm:
             parser = target.parse_args(
-                ["-i", input_file, "-o", output_file, "-c", config_file, "--debug"]
+                [
+                    "-i",
+                    input_file,
+                    "-o",
+                    output_file,
+                    "-c",
+                    config_file,
+                    "--debug",
+                    "--divert",
+                    "2,abc",
+                    "--divert",
+                    "2,def",
+                    "--divert",
+                    "3,ghi,k lm",
+                ]
             )
         self.assertEqual(parser.loglevel, logging.DEBUG)
         self.assertEqual(parser.logging_level, "DEBUG")
+        self.assertEqual(parser.divert, {2: ["abc", "def"], 3: ["ghi,k lm"]})
         self.assertEqual(
             cm.output,
             [
-                "DEBUG:root:These are the parsed arguments:\n'Namespace(config="
-                "'tests/sample_files/configuration1.xlsx', delimiter=',', "
-                "input='tests/sample_files/input1.txt', logging_level='DEBUG', "
-                "loglevel=10, output='tests/sample_files/nonexistent_test_"
-                "output.txt', overwrite_file=False, quotechar='\"', "
-                "skip_footer=0, skip_header=0)'"
+                "DEBUG:root:These are the parsed arguments:\n'Namespace("
+                "config='tests/sample_files/configuration1.xlsx', "
+                "delimiter=',', "
+                "divert={2: ['abc', 'def'], 3: ['ghi,k lm']}, "
+                "input='tests/sample_files/input1.txt', "
+                "locale='', "
+                "logging_level='DEBUG', "
+                "loglevel=10, "
+                "output='tests/sample_files/nonexistent_test_output.txt', "
+                "overwrite_file=False, "
+                "quotechar='\"', "
+                "skip_footer=0, "
+                "skip_header=0, "
+                "truncate=[])'"
             ],
         )
 
@@ -874,6 +1213,117 @@ class TestParseArgs(unittest.TestCase):
             ],
         )
 
+    def test_parse_args_truncate_valid(self):
+        """
+        Test running the script with a valid --truncate parameter
+        """
+        input_file = "tests/sample_files/input1.txt"
+        output_file = "tests/sample_files/nonexistent_test_output.txt"
+        config_file = "tests/sample_files/configuration1.xlsx"
+        parser = target.parse_args(
+            [
+                "-i",
+                input_file,
+                "-o",
+                output_file,
+                "-c",
+                config_file,
+                "--truncate",
+                "1,12,15",
+            ]
+        )
+        self.assertEqual(parser.truncate, [1, 12, 15])
+
+    def test_parse_args_truncate_invalid(self):
+        """
+        Test running the script with an invalid --truncate parameter
+        """
+        input_file = "tests/sample_files/input1.txt"
+        output_file = "tests/sample_files/nonexistent_test_output.txt"
+        config_file = "tests/sample_files/configuration1.xlsx"
+        with self.assertRaises(SystemExit) as cm1, self.assertLogs(
+            level="CRITICAL"
+        ) as cm2:
+            target.parse_args(
+                [
+                    "-i",
+                    input_file,
+                    "-o",
+                    output_file,
+                    "-c",
+                    config_file,
+                    "--truncate",
+                    "INVALID",
+                ]
+            )
+        self.assertEqual(cm1.exception.code, 25)
+        self.assertEqual(
+            cm2.output,
+            [
+                "CRITICAL:root:The `--truncate` argument must be a comma-delimited "
+                "list of numbers. Exiting..."
+            ],
+        )
+
+    def test_parse_args_divert_invalid(self):
+        """
+        Test running the script with invalid --divert arguments
+        """
+        input_file = "tests/sample_files/input1.txt"
+        output_file = "tests/sample_files/nonexistent_test_output.txt"
+        config_file = "tests/sample_files/configuration1.xlsx"
+        # Confirm the output file doesn't exist
+        if os.path.isfile(output_file):
+            os.remove(output_file)
+            self.assertFalse(os.path.isfile(output_file))
+        with self.assertRaises(SystemExit) as cm1, self.assertLogs(
+            level="CRITICAL"
+        ) as cm2:
+            target.parse_args(
+                [
+                    "-i",
+                    input_file,
+                    "-o",
+                    output_file,
+                    "-c",
+                    config_file,
+                    "--divert",
+                    "abc",
+                ]
+            )
+        self.assertEqual(cm1.exception.code, 28)
+        self.assertEqual(
+            cm2.output,
+            [
+                "CRITICAL:root:<field number> must be a number, as passed to the "
+                '`--divert` argument in the format "<field number>,<value to divert '
+                'on>" (without quotes). Exiting...'
+            ],
+        )
+        with self.assertRaises(SystemExit) as cm1, self.assertLogs(
+            level="CRITICAL"
+        ) as cm2:
+            target.parse_args(
+                [
+                    "-i",
+                    input_file,
+                    "-o",
+                    output_file,
+                    "-c",
+                    config_file,
+                    "--divert",
+                    "4",
+                ]
+            )
+        self.assertEqual(cm1.exception.code, 29)
+        self.assertEqual(
+            cm2.output,
+            [
+                'CRITICAL:root:The `--divert` argument must be formatted as "<field '
+                'number>,<value to divert on>" (without quotes). Exiting...'
+            ],
+        )
+
     def test_parse_args_version(self):
         """
         Test the --version argument
@@ -912,6 +1362,7 @@ class TestProcess(unittest.TestCase):
             skip_header,
             skip_footer,
             date_field_to_report_on,
+            "C",  # Default C locale
         )
         # Confirm the output file has been written and its content
         self.assertTrue(os.path.isfile(output_file))
@@ -946,9 +1397,9 @@ class TestProcess(unittest.TestCase):
         if os.path.isfile(output_file):
             os.remove(output_file)
             self.assertFalse(os.path.isfile(output_file))
-        input = "tests/sample_files/input1 - MMDDYYYY.txt"
+        input = "tests/sample_files/input1_dates.txt"
         output = output_file
-        config = "tests/sample_files/configuration1 - MMDDYYYY.xlsx"
+        config = "tests/sample_files/configuration1_dates.xlsx"
         delimiter = "^"
         quotechar = '"'
         skip_header = 1
@@ -963,6 +1414,68 @@ class TestProcess(unittest.TestCase):
             skip_header,
             skip_footer,
             date_field_to_report_on,
+            "C",  # Default C locale
+        )
+        # Confirm the output file has been written and its content
+        self.assertTrue(os.path.isfile(output_file))
+        with open(output_file) as f:
+            s = f.read()
+            expected_output = (
+                "0004000133034205413540000100202007312006"
+                "                                        "
+                "Leendert MOLENDIJK [90038979]           0020121231"
+                "19621030   20010322\n"
+                "0004000133034005407940000157202003051022"
+                "                                        "
+                "Leendert MOLENDIJK [90038979]           0019870201"
+                "19551106   19471115\n"
+                "0004000133034105409340022139202012252006"
+                "                                        "
+                "Leendert MOLENDIJK [90038979]           0019990722"
+                "20050103   20000131"
+            )
+            self.assertEqual(expected_output, s)
+        # Remove the output file
+        os.remove(output_file)
+        self.assertFalse(os.path.isfile(output_file))
+        self.assertEqual(num_input_rows, 3)
+        self.assertEqual(oldest_date, "20200305")
+        self.assertEqual(most_recent_date, "20201225")
+
+    def test_process_valid_divert(self):
+        """
+        Test the full process with valid arguments, including --divert
+        """
+        output_file = "tests/sample_files/nonexistent_test_output.txt"
+        diverted_output_file = "tests/sample_files/nonexistent_test_output_diverted.txt"
+        # Confirm the output file doesn't exist
+        if os.path.isfile(output_file):
+            os.remove(output_file)
+            self.assertFalse(os.path.isfile(output_file))
+        # Confirm the output file doesn't exist
+        if os.path.isfile(diverted_output_file):
+            os.remove(diverted_output_file)
+            self.assertFalse(os.path.isfile(diverted_output_file))
+        input = "tests/sample_files/input1.txt"
+        output = output_file
+        config = "tests/sample_files/configuration1.xlsx"
+        delimiter = "^"
+        quotechar = '"'
+        skip_header = 1
+        skip_footer = 1
+        date_field_to_report_on = 5
+        (num_input_rows, oldest_date, most_recent_date) = target.process(
+            input,
+            output,
+            config,
+            delimiter,
+            quotechar,
+            skip_header,
+            skip_footer,
+            date_field_to_report_on,
+            "C",  # Default C locale
+            None,
+            {4: ["1.567"]},
         )
         # Confirm the output file has been written and its content
         self.assertTrue(os.path.isfile(output_file))
@@ -972,21 +1485,108 @@ class TestProcess(unittest.TestCase):
                 "0004000133034205413540000100202007312006"
                 "                                        "
                 "Leendert MOLENDIJK [90038979]           \n"
-                "0004000133034005407940000157202003051022"
-                "                                        "
-                "Leendert MOLENDIJK [90038979]           \n"
                 "0004000133034105409340022139202012252006"
                 "                                        "
                 "Leendert MOLENDIJK [90038979]           "
             )
             self.assertEqual(expected_output, s)
-        # Remove the output file
+        # Confirm the diverted output file has been written and its content
+        self.assertTrue(os.path.isfile(diverted_output_file))
+        with open(diverted_output_file) as f:
+            s = f.read()
+            expected_output = (
+                "0004000133034005407940000157202003051022"
+                "                                        "
+                "Leendert MOLENDIJK [90038979]           "
+            )
+            self.assertEqual(expected_output, s)
+        # Remove the output files
         os.remove(output_file)
         self.assertFalse(os.path.isfile(output_file))
+        os.remove(diverted_output_file)
+        self.assertFalse(os.path.isfile(diverted_output_file))
         self.assertEqual(num_input_rows, 3)
         self.assertEqual(oldest_date, "20200305")
         self.assertEqual(most_recent_date, "20201225")
 
+    def test_process_invalid_truncate(self):
+        """
+        Test the full process with an invalid --truncate argument, value too high
+        """
+        output_file = "tests/sample_files/nonexistent_test_output.txt"
+        input = "tests/sample_files/input1.txt"
+        output = output_file
+        config = "tests/sample_files/configuration1.xlsx"
+        delimiter = "^"
+        quotechar = '"'
+        skip_header = 1
+        skip_footer = 1
+        date_field_to_report_on = 5
+        truncate = [2, 4, 255]
+        with self.assertRaises(SystemExit) as cm1, self.assertLogs(
+            level="CRITICAL"
+        ) as cm2:
+            target.process(
+                input,
+                output,
+                config,
+                delimiter,
+                quotechar,
+                skip_header,
+                skip_footer,
+                date_field_to_report_on,
+                "C",  # Default C locale
+                truncate,
+            )
+        self.assertEqual(cm1.exception.code, 26)
+        self.assertEqual(
+            cm2.output,
+            [
+                "CRITICAL:root:The value 255 passed in the `--truncate` argument is "
+                "invalid, it is higher than the 9 fields defined in the "
+                "configuration file. Exiting..."
+            ],
+        )
+
+    def test_process_invalid_divert(self):
+        """
+        Test the full process with an invalid --divert argument, value too high
+        """
+        output_file = "tests/sample_files/nonexistent_test_output.txt"
+        input = "tests/sample_files/input1.txt"
+        output = output_file
+        config = "tests/sample_files/configuration1.xlsx"
+        delimiter = "^"
+        quotechar = '"'
+        skip_header = 1
+        skip_footer = 1
+        date_field_to_report_on = 5
+        with self.assertRaises(SystemExit) as cm1, self.assertLogs(
+            level="CRITICAL"
+        ) as cm2:
+            target.process(
+                input,
+                output,
+                config,
+                delimiter,
+                quotechar,
+                skip_header,
+                skip_footer,
+                date_field_to_report_on,
+                "C",  # Default C locale
+                None,
+                {255: ["aaa"]},
+            )
+        self.assertEqual(cm1.exception.code, 30)
+        self.assertEqual(
+            cm2.output,
+            [
+                "CRITICAL:root:The value 255 passed as field ID in the `--divert` "
+                "argument is invalid, it is higher than the 9 fields defined in the "
+                "configuration file. Exiting..."
+            ],
+        )
+
 
 class TestInit(unittest.TestCase):
     def test_init_no_param(self):
@@ -1026,6 +1626,8 @@ class TestInit(unittest.TestCase):
             "1",
             "--skip-footer",
             "1",
+            "--locale",
+            "C",  # Default C locale
         ]
         target.init()
         # Confirm the output file has been written and its content
